/* WorkspaceBar Extension
 * author: Mark Bokil
 * 9/16/12
 * version 1.0.0
 * credit: gcampax, some code from Workspace Indicator 
 */

const GLib = imports.gi.GLib;
const St = imports.gi.St;
const Main = imports.ui.main;
const Util = imports.misc.util;
const Lang = imports.lang;
const Clutter = imports.gi.Clutter;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Gettext = imports.gettext.domain('gnome-shell-extensions');
const _ = Gettext.gettext;


function init(extensionMeta)
{
    return new WorkspaceBar(extensionMeta);
}

function WorkspaceBar(extensionMeta)
{
    this.init(extensionMeta);
}

function debug(str) {
    str = "--------------------> " + str;
    global.log(str);
}

WorkspaceBar.prototype = {
    
    init: function(extensionMeta) {
        this.extensionMeta = extensionMeta;
    },

    enable: function() {
        this.boxMain = new St.BoxLayout();
        this.boxMain.add_style_class_name("panelBox");
        this.currentWorkSpace = this._getCurrentWorkSpace();
            
        //Main.panel._leftBox.insert_child_at_index(this.boxMain, Main.panel._leftBox.get_children().length);
        Main.panel._centerBox.insert_child_at_index(this.boxMain,0);

        this._screenSignals = [];
        this._screenSignals.push(global.screen.connect_after('workspace-removed', Lang.bind(this,this._buildWorkSpaceBtns)));
        this._screenSignals.push(global.screen.connect_after('workspace-added', Lang.bind(this,this._buildWorkSpaceBtns)));
        this._screenSignals.push(global.screen.connect_after('workspace-switched', Lang.bind(this,this._buildWorkSpaceBtns)));

        this._buildWorkSpaceBtns();
    },

    disable: function() {
        Main.panel._centerBox.remove_actor(this.boxMain);
        this.boxMain = null;

        for (x=0; x < this._screenSignals.length; x++) {
            global.screen.disconnect(this._screenSignals[x]);
        }
        this._screenSignals = []; //truncate array
        this._screenSignals = null;
    },

    _getCurrentWorkSpace: function() {
        return global.screen.get_active_workspace().index();
    },

    _buildWorkSpaceBtns: function() {
        this._removeAllChildren(this.boxMain); //clear box container
        this.currentWorkSpace = this._getCurrentWorkSpace();
        this.buttons = []; //truncate arrays to release memory
        this.labels = [];
        let workSpaces = global.screen.n_workspaces - 1;
        let str = '';

        for (x=0; x <= workSpaces; x++) {
            str = " " + (x+1) + " ";
            if ( x == this.currentWorkSpace) {
                this.labels[x] = new St.Label({ text: _(str), style_class: "activeBtn" });
            } else {
                this.labels[x] = new St.Label({ text: _(str), style_class: "inactiveBtn" });
            }
            
            this.buttons[x] = new St.Button(); //{style_class: "panel-button"}
            this.buttons[x].set_child(this.labels[x]);
            this.buttons[x].connect('clicked', Lang.bind(this, function(actor, event) {
                this._setWorkSpace(actor.get_child().text); //use text label for workspace index
            }));

            this.buttons[x].connect('scroll-event', Lang.bind(this, this._onScrollEvent));
            this.boxMain.add_actor(this.buttons[x]);
        }
        
    },

    _removeAllChildren: function(box) {
        let children = box.get_children();

        if (children) {
            let len = children.length;

            for(x=len-1; x >= 0  ; x--) {
                box.remove_actor(children[x]);
            }
        }
        
    },

    _setWorkSpace: function(index) {
        index--; //button labels are 1,2,3, off by +1

        try {
            let possibleWorkspace = global.screen.get_workspace_by_index(index);
            possibleWorkspace.activate(global.get_current_time());
        } catch(e) {
            global.logError(e);
            return;
        }

        this._buildWorkSpaceBtns(); //refresh GUI after add,remove,switch workspace
    },

    _activateScroll : function (diff) {
        this.currentWorkSpace = this._getCurrentWorkSpace() + diff;
        let workSpaces = global.screen.n_workspaces - 1;

        if (this.currentWorkSpace < 0) this.currentWorkSpace = 0;
        if (this.currentWorkSpace > workSpaces) this.currentWorkSpace = workSpaces;

        this._setWorkSpace(this.currentWorkSpace + 1);
    },

    _onScrollEvent : function(actor, event) {
        let direction = event.get_scroll_direction();
        let offSet = 0;

        if (direction == Clutter.ScrollDirection.DOWN) {
            offSet = 1;
        } else if (direction == Clutter.ScrollDirection.UP) {
            offSet = -1;
        } else {
            return;
        }
       
        this._activateScroll(offSet);
    }
}

